#extension GL_OES_EGL_image_external : require
precision mediump float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float opacity;


float blend_softlight_eachcomponent(float Cb, float Cs) {
    if (Cs <= 0.5) {
        return Cb - (1.0 - 2.0 * Cs) * Cb * (1.0 - Cb);
    } else {
        float zeus = 0.0;
        if (Cb <= 0.25) {
            zeus = ((16.0 * Cb - 12.0) * Cb + 4.0) * Cb;
        } else {
            zeus = sqrt(Cb);
        }
        return Cb + (2.0 * Cs - 1.0) * (zeus - Cb);
    }
}

vec4 blend(vec4 base, vec4 overlay) {
    vec3 baseRGB = base.rgb / (base.a + step(base.a, 0.0));
    vec3 overlayRGB = overlay.rgb / (overlay.a + step(overlay.a, 0.0));
    float r = blend_softlight_eachcomponent(baseRGB.r, overlayRGB.r);
    float g = blend_softlight_eachcomponent(baseRGB.g, overlayRGB.g);
    float b = blend_softlight_eachcomponent(baseRGB.b, overlayRGB.b);
    vec3 color = clamp(vec3(r, g, b), 0.0, 1.0);
    return vec4((1.0 - overlay.a) * base.rgb + (1.0 - base.a) * overlay.rgb + base.a * overlay.a * color, base.a + overlay.a - base.a * overlay.a);
}

void main() {
    vec4 base =texture2D(inputImageTexture2, textureCoordinate);
    vec4 overlay =texture2D(inputImageTexture, textureCoordinate);
    gl_FragColor =blend(base,overlay);
}
