precision highp float;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp vec2 referPoint;
uniform highp vec2 point1;
uniform highp vec2 point2;
uniform highp float radius;
uniform highp float ratio;
uniform bool eraseMode;

highp float PointToSegDist(vec2 p, mediump vec2 p1, mediump vec2 p2) {
    p.y = p.y / ratio;
    p1.y = p1.y / ratio;
    p2.y = p2.y / ratio;
    float cross = (p2.x - p1.x) * (p.x - p1.x) + (p2.y - p1.y) * (p.y - p1.y);
    if (cross <= 0.0) return distance(p, p1);

    float d2 = (p2.x - p1.x) * (p2.x - p1.x) + (p2.y - p1.y) * (p2.y - p1.y);
    if (cross >= d2) return distance(p, p2);

    float r = cross / d2;
    vec2 pp = vec2(p1.x + (p2.x - p1.x) * r, p1.y + (p2.y - p1.y) * r);
    return distance(p, pp);
}

void main() {
    vec4 texture1Color = texture2D(inputImageTexture, textureCoordinate);
    float calcRadius = PointToSegDist(textureCoordinate, point1, point2);
    if (calcRadius <= radius) {
        vec4 originColor = texture2D(inputImageTexture2, textureCoordinate);
        if (!eraseMode) {
            float alphaMulti2 = smoothstep(radius * 1.0, radius, calcRadius);
            float maxalpha = max(texture1Color.a, originColor.a * (1.0 - alphaMulti2));
            gl_FragColor = vec4(originColor.rgb, maxalpha);
            return;
        }
        vec4 texture1EraserColor = texture2D(inputImageTexture2, referPoint);
        float colorDistance = distance(texture1EraserColor.rgb, originColor.rgb);
        if (colorDistance < 0.26) {
            float alphaMulti2 = smoothstep(0.0, radius, calcRadius);
            // 会导致花边
//            gl_FragColor = vec4(texture1Color.rgb, texture1Color.a * alphaMulti2);
            gl_FragColor = texture1Color * alphaMulti2;
            return;
        }
        // 擦除噪点
        int tcount = 0;
        float offset1 = 0.1 * radius;
        float offset2 = 0.707 * offset1;
        float threthold = 0.1;
        int limitCount = 5;
        vec2 coord = vec2(textureCoordinate.x - offset1, textureCoordinate.y);
        float ap = texture2D(inputImageTexture, coord).a;
        if (ap < threthold) {
            tcount = tcount + 1;
        }
        coord = vec2(textureCoordinate.x - offset2, textureCoordinate.y + offset2);
        ap = texture2D(inputImageTexture, coord).a;
        if (ap < threthold) {
            tcount = tcount + 1;
        }
        coord = vec2(textureCoordinate.x, textureCoordinate.y + offset1);
        ap = texture2D(inputImageTexture, coord).a;
        if (ap < threthold) {
            tcount = tcount + 1;
        }
        coord = vec2(textureCoordinate.x + offset2, textureCoordinate.y + offset2);
        ap = texture2D(inputImageTexture, coord).a;
        if (ap < threthold) {
            tcount = tcount + 1;
        }
        coord = vec2(textureCoordinate.x + offset1, textureCoordinate.y);
        ap = texture2D(inputImageTexture, coord).a;
        if (ap < threthold) {
            tcount = tcount + 1;
        }
        if (tcount >= limitCount){
            gl_FragColor = vec4(texture1Color.rgb, 0.0);
            return;
        }
        coord = vec2(textureCoordinate.x + offset2, textureCoordinate.y - offset2);
        ap = texture2D(inputImageTexture, coord).a;
        if (ap < threthold) {
            tcount = tcount + 1;
        }
        if (tcount >= limitCount){
            gl_FragColor = vec4(texture1Color.rgb, 0.0);
            return;
        }
        coord = vec2(textureCoordinate.x, textureCoordinate.y - offset1);
        ap = texture2D(inputImageTexture, coord).a;
        if (ap < threthold) {
            tcount = tcount + 1;
        }
        if (tcount >= limitCount){
            gl_FragColor = vec4(texture1Color.rgb, 0.0);
            return;
        }
        coord = vec2(textureCoordinate.x - offset2, textureCoordinate.y - offset2);
        ap = texture2D(inputImageTexture, coord).a;
        if (ap < threthold) {
            tcount = tcount + 1;
        }
        if (tcount >= limitCount){
            gl_FragColor = vec4(texture1Color.rgb, 0.0);
            return;
        }
    }
    gl_FragColor = texture1Color;
}
